# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Public API for tf.raw_ops namespace.
"""

from __future__ import print_function as _print_function

from tensorflow.python import Abs
from tensorflow.python import AccumulateNV2
from tensorflow.python import AccumulatorApplyGradient
from tensorflow.python import AccumulatorNumAccumulated
from tensorflow.python import AccumulatorSetGlobalStep
from tensorflow.python import AccumulatorTakeGradient
from tensorflow.python import Acos
from tensorflow.python import Acosh
from tensorflow.python import Add
from tensorflow.python import AddManySparseToTensorsMap
from tensorflow.python import AddN
from tensorflow.python import AddSparseToTensorsMap
from tensorflow.python import AddV2
from tensorflow.python import All
from tensorflow.python import Angle
from tensorflow.python import Any
from tensorflow.python import ApproximateEqual
from tensorflow.python import ArgMax
from tensorflow.python import ArgMin
from tensorflow.python import AsString
from tensorflow.python import Asin
from tensorflow.python import Asinh
from tensorflow.python import Atan
from tensorflow.python import Atan2
from tensorflow.python import Atanh
from tensorflow.python import BarrierClose
from tensorflow.python import BarrierIncompleteSize
from tensorflow.python import BarrierInsertMany
from tensorflow.python import BarrierReadySize
from tensorflow.python import BarrierTakeMany
from tensorflow.python import Batch
from tensorflow.python import BatchCholesky
from tensorflow.python import BatchCholeskyGrad
from tensorflow.python import BatchFunction
from tensorflow.python import BatchMatMul
from tensorflow.python import BatchMatMulV2
from tensorflow.python import BatchMatrixBandPart
from tensorflow.python import BatchMatrixDeterminant
from tensorflow.python import BatchMatrixDiag
from tensorflow.python import BatchMatrixDiagPart
from tensorflow.python import BatchMatrixInverse
from tensorflow.python import BatchMatrixSetDiag
from tensorflow.python import BatchMatrixSolve
from tensorflow.python import BatchMatrixSolveLs
from tensorflow.python import BatchMatrixTriangularSolve
from tensorflow.python import BatchSelfAdjointEig
from tensorflow.python import BatchSelfAdjointEigV2
from tensorflow.python import BatchSvd
from tensorflow.python import BatchToSpace
from tensorflow.python import BatchToSpaceND
from tensorflow.python import BesselI0e
from tensorflow.python import BesselI1e
from tensorflow.python import Betainc
from tensorflow.python import Bincount
from tensorflow.python import Bitcast
from tensorflow.python import BroadcastArgs
from tensorflow.python import BroadcastGradientArgs
from tensorflow.python import BroadcastTo
from tensorflow.python import Bucketize
from tensorflow.python import Cast
from tensorflow.python import Ceil
from tensorflow.python import CheckNumerics
from tensorflow.python import Cholesky
from tensorflow.python import CholeskyGrad
from tensorflow.python import ClipByValue
from tensorflow.python import CompareAndBitpack
from tensorflow.python import Complex
from tensorflow.python import ComplexAbs
from tensorflow.python import Concat
from tensorflow.python import ConcatOffset
from tensorflow.python import ConcatV2
from tensorflow.python import Conj
from tensorflow.python import ConjugateTranspose
from tensorflow.python import Const
from tensorflow.python import Cos
from tensorflow.python import Cosh
from tensorflow.python import Cross
from tensorflow.python import Cumprod
from tensorflow.python import Cumsum
from tensorflow.python import DebugGradientIdentity
from tensorflow.python import DebugGradientRefIdentity
from tensorflow.python import DecodeBase64
from tensorflow.python import DecodeCSV
from tensorflow.python import DecodeCompressed
from tensorflow.python import DecodeJSONExample
from tensorflow.python import DecodePaddedRaw
from tensorflow.python import DecodeRaw
from tensorflow.python import DeepCopy
from tensorflow.python import DeleteSessionTensor
from tensorflow.python import DepthToSpace
from tensorflow.python import Dequantize
from tensorflow.python import DeserializeManySparse
from tensorflow.python import DeserializeSparse
from tensorflow.python import Diag
from tensorflow.python import DiagPart
from tensorflow.python import Digamma
from tensorflow.python import Div
from tensorflow.python import DivNoNan
from tensorflow.python import DynamicPartition
from tensorflow.python import DynamicStitch
from tensorflow.python import EditDistance
from tensorflow.python import Empty
from tensorflow.python import EncodeBase64
from tensorflow.python import EnsureShape
from tensorflow.python import Equal
from tensorflow.python import Erf
from tensorflow.python import Erfc
from tensorflow.python import EuclideanNorm
from tensorflow.python import Exp
from tensorflow.python import ExpandDims
from tensorflow.python import Expm1
from tensorflow.python import ExtractImagePatches
from tensorflow.python import ExtractVolumePatches
from tensorflow.python import FIFOQueueV2
from tensorflow.python import FakeQuantWithMinMaxArgs
from tensorflow.python import FakeQuantWithMinMaxArgsGradient
from tensorflow.python import FakeQuantWithMinMaxVars
from tensorflow.python import FakeQuantWithMinMaxVarsGradient
from tensorflow.python import FakeQuantWithMinMaxVarsPerChannel
from tensorflow.python import FakeQuantWithMinMaxVarsPerChannelGradient
from tensorflow.python import FakeQueue
from tensorflow.python import Fill
from tensorflow.python import Fingerprint
from tensorflow.python import FixedLengthRecordReaderV2
from tensorflow.python import Floor
from tensorflow.python import FloorDiv
from tensorflow.python import FloorMod
from tensorflow.python import Gather
from tensorflow.python import GatherNd
from tensorflow.python import GatherV2
from tensorflow.python import GetSessionHandle
from tensorflow.python import GetSessionHandleV2
from tensorflow.python import GetSessionTensor
from tensorflow.python import Greater
from tensorflow.python import GreaterEqual
from tensorflow.python import GuaranteeConst
from tensorflow.python import HistogramFixedWidth
from tensorflow.python import IdentityN
from tensorflow.python import IdentityReaderV2
from tensorflow.python import Igamma
from tensorflow.python import IgammaGradA
from tensorflow.python import Igammac
from tensorflow.python import Imag
from tensorflow.python import ImmutableConst
from tensorflow.python import InplaceAdd
from tensorflow.python import InplaceSub
from tensorflow.python import InplaceUpdate
from tensorflow.python import Inv
from tensorflow.python import InvGrad
from tensorflow.python import InvertPermutation
from tensorflow.python import IsFinite
from tensorflow.python import IsInf
from tensorflow.python import IsNan
from tensorflow.python import Less
from tensorflow.python import LessEqual
from tensorflow.python import Lgamma
from tensorflow.python import LinSpace
from tensorflow.python import ListDiff
from tensorflow.python import Log
from tensorflow.python import Log1p
from tensorflow.python import LogMatrixDeterminant
from tensorflow.python import LogicalAnd
from tensorflow.python import LogicalNot
from tensorflow.python import LogicalOr
from tensorflow.python import LowerBound
from tensorflow.python import Lu
from tensorflow.python import MapClear
from tensorflow.python import MapIncompleteSize
from tensorflow.python import MapPeek
from tensorflow.python import MapSize
from tensorflow.python import MapStage
from tensorflow.python import MapUnstage
from tensorflow.python import MapUnstageNoKey
from tensorflow.python import MatMul
from tensorflow.python import MatchingFiles
from tensorflow.python import MatrixBandPart
from tensorflow.python import MatrixDeterminant
from tensorflow.python import MatrixDiag
from tensorflow.python import MatrixDiagPart
from tensorflow.python import MatrixExponential
from tensorflow.python import MatrixInverse
from tensorflow.python import MatrixLogarithm
from tensorflow.python import MatrixSetDiag
from tensorflow.python import MatrixSolve
from tensorflow.python import MatrixSolveLs
from tensorflow.python import MatrixSquareRoot
from tensorflow.python import MatrixTriangularSolve
from tensorflow.python import Max
from tensorflow.python import Maximum
from tensorflow.python import Mean
from tensorflow.python import MergeV2Checkpoints
from tensorflow.python import Min
from tensorflow.python import Minimum
from tensorflow.python import MirrorPad
from tensorflow.python import MirrorPadGrad
from tensorflow.python import Mod
from tensorflow.python import Mul
from tensorflow.python import MulNoNan
from tensorflow.python import Multinomial
from tensorflow.python import Neg
from tensorflow.python import NextAfter
from tensorflow.python import NotEqual
from tensorflow.python import OneHot
from tensorflow.python import OnesLike
from tensorflow.python import OrderedMapClear
from tensorflow.python import OrderedMapIncompleteSize
from tensorflow.python import OrderedMapPeek
from tensorflow.python import OrderedMapSize
from tensorflow.python import OrderedMapStage
from tensorflow.python import OrderedMapUnstage
from tensorflow.python import OrderedMapUnstageNoKey
from tensorflow.python import Pack
from tensorflow.python import Pad
from tensorflow.python import PadV2
from tensorflow.python import PaddingFIFOQueueV2
from tensorflow.python import ParallelConcat
from tensorflow.python import ParallelDynamicStitch
from tensorflow.python import ParameterizedTruncatedNormal
from tensorflow.python import ParseExample
from tensorflow.python import ParseSequenceExample
from tensorflow.python import ParseSingleExample
from tensorflow.python import ParseSingleSequenceExample
from tensorflow.python import ParseTensor
from tensorflow.python import Placeholder
from tensorflow.python import PlaceholderV2
from tensorflow.python import PlaceholderWithDefault
from tensorflow.python import Polygamma
from tensorflow.python import Pow
from tensorflow.python import PreventGradient
from tensorflow.python import PriorityQueueV2
from tensorflow.python import Prod
from tensorflow.python import Qr
from tensorflow.python import QuantizeAndDequantize
from tensorflow.python import QuantizeAndDequantizeV2
from tensorflow.python import QuantizeAndDequantizeV3
from tensorflow.python import QuantizeDownAndShrinkRange
from tensorflow.python import QuantizeV2
from tensorflow.python import QuantizedAdd
from tensorflow.python import QuantizedConcat
from tensorflow.python import QuantizedInstanceNorm
from tensorflow.python import QuantizedMatMul
from tensorflow.python import QuantizedMul
from tensorflow.python import QuantizedReshape
from tensorflow.python import QueueClose
from tensorflow.python import QueueCloseV2
from tensorflow.python import QueueDequeue
from tensorflow.python import QueueDequeueMany
from tensorflow.python import QueueDequeueManyV2
from tensorflow.python import QueueDequeueUpTo
from tensorflow.python import QueueDequeueUpToV2
from tensorflow.python import QueueDequeueV2
from tensorflow.python import QueueEnqueue
from tensorflow.python import QueueEnqueueMany
from tensorflow.python import QueueEnqueueManyV2
from tensorflow.python import QueueEnqueueV2
from tensorflow.python import QueueIsClosed
from tensorflow.python import QueueIsClosedV2
from tensorflow.python import QueueSize
from tensorflow.python import QueueSizeV2
from tensorflow.python import RandomGamma
from tensorflow.python import RandomGammaGrad
from tensorflow.python import RandomPoisson
from tensorflow.python import RandomPoissonV2
from tensorflow.python import RandomShuffle
from tensorflow.python import RandomShuffleQueueV2
from tensorflow.python import RandomStandardNormal
from tensorflow.python import RandomUniform
from tensorflow.python import RandomUniformInt
from tensorflow.python import Range
from tensorflow.python import Rank
from tensorflow.python import ReadFile
from tensorflow.python import ReaderNumRecordsProduced
from tensorflow.python import ReaderNumRecordsProducedV2
from tensorflow.python import ReaderNumWorkUnitsCompleted
from tensorflow.python import ReaderNumWorkUnitsCompletedV2
from tensorflow.python import ReaderRead
from tensorflow.python import ReaderReadUpTo
from tensorflow.python import ReaderReadUpToV2
from tensorflow.python import ReaderReadV2
from tensorflow.python import ReaderReset
from tensorflow.python import ReaderResetV2
from tensorflow.python import ReaderRestoreState
from tensorflow.python import ReaderRestoreStateV2
from tensorflow.python import ReaderSerializeState
from tensorflow.python import ReaderSerializeStateV2
from tensorflow.python import Real
from tensorflow.python import RealDiv
from tensorflow.python import Reciprocal
from tensorflow.python import ReciprocalGrad
from tensorflow.python import ReduceJoin
from tensorflow.python import RefIdentity
from tensorflow.python import RegexFullMatch
from tensorflow.python import RegexReplace
from tensorflow.python import RequantizationRange
from tensorflow.python import RequantizationRangePerChannel
from tensorflow.python import Requantize
from tensorflow.python import RequantizePerChannel
from tensorflow.python import Reshape
from tensorflow.python import ResourceStridedSliceAssign
from tensorflow.python import Restore
from tensorflow.python import RestoreSlice
from tensorflow.python import RestoreV2
from tensorflow.python import Reverse
from tensorflow.python import ReverseSequence
from tensorflow.python import ReverseV2
from tensorflow.python import Rint
from tensorflow.python import Round
from tensorflow.python import Rsqrt
from tensorflow.python import RsqrtGrad
from tensorflow.python import Save
from tensorflow.python import SaveSlices
from tensorflow.python import SaveV2
from tensorflow.python import ScatterNd
from tensorflow.python import ScatterNdNonAliasingAdd
from tensorflow.python import SegmentMax
from tensorflow.python import SegmentMean
from tensorflow.python import SegmentMin
from tensorflow.python import SegmentProd
from tensorflow.python import SegmentSum
from tensorflow.python import Select
from tensorflow.python import SelectV2
from tensorflow.python import SelfAdjointEig
from tensorflow.python import SelfAdjointEigV2
from tensorflow.python import SerializeManySparse
from tensorflow.python import SerializeSparse
from tensorflow.python import SerializeTensor
from tensorflow.python import Shape
from tensorflow.python import ShapeN
from tensorflow.python import ShardedFilename
from tensorflow.python import ShardedFilespec
from tensorflow.python import Sigmoid
from tensorflow.python import SigmoidGrad
from tensorflow.python import Sign
from tensorflow.python import Sin
from tensorflow.python import Sinh
from tensorflow.python import Size
from tensorflow.python import Slice
from tensorflow.python import Snapshot
from tensorflow.python import SpaceToBatch
from tensorflow.python import SpaceToBatchND
from tensorflow.python import SpaceToDepth
from tensorflow.python import SparseAccumulatorApplyGradient
from tensorflow.python import SparseAccumulatorTakeGradient
from tensorflow.python import SparseAdd
from tensorflow.python import SparseAddGrad
from tensorflow.python import SparseConcat
from tensorflow.python import SparseCross
from tensorflow.python import SparseDenseCwiseAdd
from tensorflow.python import SparseDenseCwiseDiv
from tensorflow.python import SparseDenseCwiseMul
from tensorflow.python import SparseFillEmptyRows
from tensorflow.python import SparseFillEmptyRowsGrad
from tensorflow.python import SparseMatMul
from tensorflow.python import SparseReduceMax
from tensorflow.python import SparseReduceMaxSparse
from tensorflow.python import SparseReduceSum
from tensorflow.python import SparseReduceSumSparse
from tensorflow.python import SparseReorder
from tensorflow.python import SparseReshape
from tensorflow.python import SparseSegmentMean
from tensorflow.python import SparseSegmentMeanGrad
from tensorflow.python import SparseSegmentMeanWithNumSegments
from tensorflow.python import SparseSegmentSqrtN
from tensorflow.python import SparseSegmentSqrtNGrad
from tensorflow.python import SparseSegmentSqrtNWithNumSegments
from tensorflow.python import SparseSegmentSum
from tensorflow.python import SparseSegmentSumWithNumSegments
from tensorflow.python import SparseSlice
from tensorflow.python import SparseSliceGrad
from tensorflow.python import SparseSoftmax
from tensorflow.python import SparseSparseMaximum
from tensorflow.python import SparseSparseMinimum
from tensorflow.python import SparseSplit
from tensorflow.python import SparseTensorDenseAdd
from tensorflow.python import SparseTensorDenseMatMul
from tensorflow.python import SparseToDense
from tensorflow.python import Split
from tensorflow.python import SplitV
from tensorflow.python import Sqrt
from tensorflow.python import SqrtGrad
from tensorflow.python import Square
from tensorflow.python import SquaredDifference
from tensorflow.python import Squeeze
from tensorflow.python import Stack
from tensorflow.python import StackClose
from tensorflow.python import StackCloseV2
from tensorflow.python import StackPop
from tensorflow.python import StackPopV2
from tensorflow.python import StackPush
from tensorflow.python import StackPushV2
from tensorflow.python import StackV2
from tensorflow.python import Stage
from tensorflow.python import StageClear
from tensorflow.python import StagePeek
from tensorflow.python import StageSize
from tensorflow.python import StaticRegexFullMatch
from tensorflow.python import StaticRegexReplace
from tensorflow.python import StopGradient
from tensorflow.python import StridedSlice
from tensorflow.python import StridedSliceAssign
from tensorflow.python import StridedSliceGrad
from tensorflow.python import StringFormat
from tensorflow.python import StringJoin
from tensorflow.python import StringLength
from tensorflow.python import StringLower
from tensorflow.python import StringSplit
from tensorflow.python import StringSplitV2
from tensorflow.python import StringStrip
from tensorflow.python import StringToHashBucket
from tensorflow.python import StringToHashBucketFast
from tensorflow.python import StringToHashBucketStrong
from tensorflow.python import StringToNumber
from tensorflow.python import StringUpper
from tensorflow.python import Sub
from tensorflow.python import Substr
from tensorflow.python import Sum
from tensorflow.python import Svd
from tensorflow.python import TFRecordReaderV2
from tensorflow.python import TakeManySparseFromTensorsMap
from tensorflow.python import Tan
from tensorflow.python import Tanh
from tensorflow.python import TanhGrad
from tensorflow.python import TensorArrayClose
from tensorflow.python import TensorArrayCloseV2
from tensorflow.python import TensorArrayCloseV3
from tensorflow.python import TensorArrayConcat
from tensorflow.python import TensorArrayConcatV2
from tensorflow.python import TensorArrayConcatV3
from tensorflow.python import TensorArrayGather
from tensorflow.python import TensorArrayGatherV2
from tensorflow.python import TensorArrayGatherV3
from tensorflow.python import TensorArrayGrad
from tensorflow.python import TensorArrayGradV2
from tensorflow.python import TensorArrayGradV3
from tensorflow.python import TensorArrayGradWithShape
from tensorflow.python import TensorArrayPack
from tensorflow.python import TensorArrayRead
from tensorflow.python import TensorArrayReadV2
from tensorflow.python import TensorArrayReadV3
from tensorflow.python import TensorArrayScatter
from tensorflow.python import TensorArrayScatterV2
from tensorflow.python import TensorArrayScatterV3
from tensorflow.python import TensorArraySize
from tensorflow.python import TensorArraySizeV2
from tensorflow.python import TensorArraySizeV3
from tensorflow.python import TensorArraySplit
from tensorflow.python import TensorArraySplitV2
from tensorflow.python import TensorArraySplitV3
from tensorflow.python import TensorArrayUnpack
from tensorflow.python import TensorArrayV2
from tensorflow.python import TensorArrayV3
from tensorflow.python import TensorArrayWrite
from tensorflow.python import TensorArrayWriteV2
from tensorflow.python import TensorArrayWriteV3
from tensorflow.python import TensorScatterAdd
from tensorflow.python import TensorScatterSub
from tensorflow.python import TensorScatterUpdate
from tensorflow.python import TensorStridedSliceUpdate
from tensorflow.python import TextLineReaderV2
from tensorflow.python import Tile
from tensorflow.python import TileGrad
from tensorflow.python import Transpose
from tensorflow.python import TridiagonalMatMul
from tensorflow.python import TridiagonalSolve
from tensorflow.python import TruncateDiv
from tensorflow.python import TruncateMod
from tensorflow.python import TruncatedNormal
from tensorflow.python import Unbatch
from tensorflow.python import UnbatchGrad
from tensorflow.python import UnicodeDecode
from tensorflow.python import UnicodeDecodeWithOffsets
from tensorflow.python import UnicodeEncode
from tensorflow.python import UnicodeScript
from tensorflow.python import UnicodeTranscode
from tensorflow.python import Unique
from tensorflow.python import UniqueV2
from tensorflow.python import UniqueWithCounts
from tensorflow.python import UniqueWithCountsV2
from tensorflow.python import Unpack
from tensorflow.python import UnravelIndex
from tensorflow.python import UnsortedSegmentMax
from tensorflow.python import UnsortedSegmentMin
from tensorflow.python import UnsortedSegmentProd
from tensorflow.python import UnsortedSegmentSum
from tensorflow.python import Unstage
from tensorflow.python import UpperBound
from tensorflow.python import Where
from tensorflow.python import WholeFileReaderV2
from tensorflow.python import WriteFile
from tensorflow.python import Xdivy
from tensorflow.python import Xlogy
from tensorflow.python import ZerosLike
from tensorflow.python import Zeta
from tensorflow.python.ops.array_ops import Identity
from tensorflow.python.ops.bitwise_ops import BitwiseAnd
from tensorflow.python.ops.bitwise_ops import BitwiseOr
from tensorflow.python.ops.bitwise_ops import BitwiseXor
from tensorflow.python.ops.bitwise_ops import Invert
from tensorflow.python.ops.bitwise_ops import LeftShift
from tensorflow.python.ops.bitwise_ops import PopulationCount
from tensorflow.python.ops.bitwise_ops import RightShift
from tensorflow.python.ops.control_flow_grad import Abort
from tensorflow.python.ops.control_flow_grad import ControlTrigger
from tensorflow.python.ops.control_flow_grad import Enter
from tensorflow.python.ops.control_flow_grad import Exit
from tensorflow.python.ops.control_flow_grad import LoopCond
from tensorflow.python.ops.control_flow_grad import Merge
from tensorflow.python.ops.control_flow_grad import NextIteration
from tensorflow.python.ops.control_flow_grad import NoOp
from tensorflow.python.ops.control_flow_grad import RefEnter
from tensorflow.python.ops.control_flow_grad import RefExit
from tensorflow.python.ops.control_flow_grad import RefMerge
from tensorflow.python.ops.control_flow_grad import RefNextIteration
from tensorflow.python.ops.control_flow_grad import RefSelect
from tensorflow.python.ops.control_flow_grad import RefSwitch
from tensorflow.python.ops.control_flow_grad import Switch
from tensorflow.python.ops.data_flow_ops import TensorArray
from tensorflow.python.ops.gen_audio_ops import AudioSpectrogram
from tensorflow.python.ops.gen_audio_ops import DecodeWav
from tensorflow.python.ops.gen_audio_ops import EncodeWav
from tensorflow.python.ops.gen_audio_ops import Mfcc
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesAggregateStats
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesBucketize
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesCalculateBestFeatureSplit
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesCalculateBestGainsPerFeature
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesCenterBias
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesCreateEnsemble
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesCreateQuantileStreamResource
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesDeserializeEnsemble
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesEnsembleResourceHandleOp
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesExampleDebugOutputs
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesGetEnsembleStates
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesMakeQuantileSummaries
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesMakeStatsSummary
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesPredict
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesQuantileStreamResourceAddSummaries
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesQuantileStreamResourceDeserialize
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesQuantileStreamResourceFlush
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesQuantileStreamResourceGetBucketBoundaries
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesQuantileStreamResourceHandleOp
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesSerializeEnsemble
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesTrainingPredict
from tensorflow.python.ops.gen_boosted_trees_ops import BoostedTreesUpdateEnsemble
from tensorflow.python.ops.gen_boosted_trees_ops import IsBoostedTreesEnsembleInitialized
from tensorflow.python.ops.gen_boosted_trees_ops import IsBoostedTreesQuantileStreamResourceInitialized
from tensorflow.python.ops.gen_candidate_sampling_ops import AllCandidateSampler
from tensorflow.python.ops.gen_candidate_sampling_ops import ComputeAccidentalHits
from tensorflow.python.ops.gen_candidate_sampling_ops import FixedUnigramCandidateSampler
from tensorflow.python.ops.gen_candidate_sampling_ops import LearnedUnigramCandidateSampler
from tensorflow.python.ops.gen_candidate_sampling_ops import LogUniformCandidateSampler
from tensorflow.python.ops.gen_candidate_sampling_ops import ThreadUnsafeUnigramCandidateSampler
from tensorflow.python.ops.gen_candidate_sampling_ops import UniformCandidateSampler
from tensorflow.python.ops.gen_checkpoint_ops import GenerateVocabRemapping
from tensorflow.python.ops.gen_checkpoint_ops import LoadAndRemapMatrix
from tensorflow.python.ops.gen_collective_ops import CollectiveBcastRecv
from tensorflow.python.ops.gen_collective_ops import CollectiveBcastSend
from tensorflow.python.ops.gen_collective_ops import CollectiveGather
from tensorflow.python.ops.gen_collective_ops import CollectiveReduce
from tensorflow.python.ops.gen_ctc_ops import CTCBeamSearchDecoder
from tensorflow.python.ops.gen_ctc_ops import CTCGreedyDecoder
from tensorflow.python.ops.gen_ctc_ops import CTCLoss
from tensorflow.python.ops.gen_cudnn_rnn_ops import CudnnRNN
from tensorflow.python.ops.gen_cudnn_rnn_ops import CudnnRNNBackprop
from tensorflow.python.ops.gen_cudnn_rnn_ops import CudnnRNNBackpropV2
from tensorflow.python.ops.gen_cudnn_rnn_ops import CudnnRNNBackpropV3
from tensorflow.python.ops.gen_cudnn_rnn_ops import CudnnRNNCanonicalToParams
from tensorflow.python.ops.gen_cudnn_rnn_ops import CudnnRNNParamsSize
from tensorflow.python.ops.gen_cudnn_rnn_ops import CudnnRNNParamsToCanonical
from tensorflow.python.ops.gen_cudnn_rnn_ops import CudnnRNNV2
from tensorflow.python.ops.gen_cudnn_rnn_ops import CudnnRNNV3
from tensorflow.python.ops.gen_data_flow_ops import Barrier
from tensorflow.python.ops.gen_data_flow_ops import ConditionalAccumulator
from tensorflow.python.ops.gen_data_flow_ops import FIFOQueue
from tensorflow.python.ops.gen_data_flow_ops import PaddingFIFOQueue
from tensorflow.python.ops.gen_data_flow_ops import PriorityQueue
from tensorflow.python.ops.gen_data_flow_ops import RandomShuffleQueue
from tensorflow.python.ops.gen_data_flow_ops import RecordInput
from tensorflow.python.ops.gen_data_flow_ops import SparseConditionalAccumulator
from tensorflow.python.ops.gen_dataset_ops import AnonymousIterator
from tensorflow.python.ops.gen_dataset_ops import AnonymousIteratorV2
from tensorflow.python.ops.gen_dataset_ops import BatchDataset
from tensorflow.python.ops.gen_dataset_ops import BatchDatasetV2
from tensorflow.python.ops.gen_dataset_ops import CacheDataset
from tensorflow.python.ops.gen_dataset_ops import ConcatenateDataset
from tensorflow.python.ops.gen_dataset_ops import DatasetToGraph
from tensorflow.python.ops.gen_dataset_ops import DatasetToSingleElement
from tensorflow.python.ops.gen_dataset_ops import DeleteIterator
from tensorflow.python.ops.gen_dataset_ops import DeserializeIterator
from tensorflow.python.ops.gen_dataset_ops import FilterByLastComponentDataset
from tensorflow.python.ops.gen_dataset_ops import FilterDataset
from tensorflow.python.ops.gen_dataset_ops import FixedLengthRecordDataset
from tensorflow.python.ops.gen_dataset_ops import FixedLengthRecordDatasetV2
from tensorflow.python.ops.gen_dataset_ops import FlatMapDataset
from tensorflow.python.ops.gen_dataset_ops import GeneratorDataset
from tensorflow.python.ops.gen_dataset_ops import InterleaveDataset
from tensorflow.python.ops.gen_dataset_ops import Iterator
from tensorflow.python.ops.gen_dataset_ops import IteratorFromStringHandle
from tensorflow.python.ops.gen_dataset_ops import IteratorFromStringHandleV2
from tensorflow.python.ops.gen_dataset_ops import IteratorGetNext
from tensorflow.python.ops.gen_dataset_ops import IteratorGetNextAsOptional
from tensorflow.python.ops.gen_dataset_ops import IteratorGetNextSync
from tensorflow.python.ops.gen_dataset_ops import IteratorToStringHandle
from tensorflow.python.ops.gen_dataset_ops import IteratorV2
from tensorflow.python.ops.gen_dataset_ops import MakeIterator
from tensorflow.python.ops.gen_dataset_ops import MapDataset
from tensorflow.python.ops.gen_dataset_ops import MapDefun
from tensorflow.python.ops.gen_dataset_ops import ModelDataset
from tensorflow.python.ops.gen_dataset_ops import MultiDeviceIterator
from tensorflow.python.ops.gen_dataset_ops import MultiDeviceIteratorFromStringHandle
from tensorflow.python.ops.gen_dataset_ops import MultiDeviceIteratorGetNextFromShard
from tensorflow.python.ops.gen_dataset_ops import MultiDeviceIteratorInit
from tensorflow.python.ops.gen_dataset_ops import MultiDeviceIteratorToStringHandle
from tensorflow.python.ops.gen_dataset_ops import OneShotIterator
from tensorflow.python.ops.gen_dataset_ops import OptimizeDataset
from tensorflow.python.ops.gen_dataset_ops import OptionalFromValue
from tensorflow.python.ops.gen_dataset_ops import OptionalGetValue
from tensorflow.python.ops.gen_dataset_ops import OptionalHasValue
from tensorflow.python.ops.gen_dataset_ops import OptionalNone
from tensorflow.python.ops.gen_dataset_ops import PaddedBatchDataset
from tensorflow.python.ops.gen_dataset_ops import PaddedBatchDatasetV2
from tensorflow.python.ops.gen_dataset_ops import ParallelInterleaveDatasetV2
from tensorflow.python.ops.gen_dataset_ops import ParallelMapDataset
from tensorflow.python.ops.gen_dataset_ops import PrefetchDataset
from tensorflow.python.ops.gen_dataset_ops import RangeDataset
from tensorflow.python.ops.gen_dataset_ops import ReduceDataset
from tensorflow.python.ops.gen_dataset_ops import RepeatDataset
from tensorflow.python.ops.gen_dataset_ops import SerializeIterator
from tensorflow.python.ops.gen_dataset_ops import ShardDataset
from tensorflow.python.ops.gen_dataset_ops import ShuffleAndRepeatDataset
from tensorflow.python.ops.gen_dataset_ops import ShuffleDataset
from tensorflow.python.ops.gen_dataset_ops import SkipDataset
from tensorflow.python.ops.gen_dataset_ops import SparseTensorSliceDataset
from tensorflow.python.ops.gen_dataset_ops import TFRecordDataset
from tensorflow.python.ops.gen_dataset_ops import TakeDataset
from tensorflow.python.ops.gen_dataset_ops import TensorDataset
from tensorflow.python.ops.gen_dataset_ops import TensorSliceDataset
from tensorflow.python.ops.gen_dataset_ops import TextLineDataset
from tensorflow.python.ops.gen_dataset_ops import UnwrapDatasetVariant
from tensorflow.python.ops.gen_dataset_ops import WindowDataset
from tensorflow.python.ops.gen_dataset_ops import WrapDatasetVariant
from tensorflow.python.ops.gen_dataset_ops import ZipDataset
from tensorflow.python.ops.gen_decode_proto_ops import DecodeProtoV2
from tensorflow.python.ops.gen_encode_proto_ops import EncodeProto
from tensorflow.python.ops.gen_experimental_dataset_ops import ChooseFastestBranchDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalAssertNextDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalAutoShardDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalBytesProducedStatsDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalCSVDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalChooseFastestDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalDatasetCardinality
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalDatasetToTFRecord
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalDenseToSparseBatchDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalDirectedInterleaveDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalGroupByReducerDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalGroupByWindowDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalIdentityIndexedDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalIgnoreErrorsDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalIndexedDatasetGet
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalIndexedDatasetMaterialize
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalIteratorGetDevice
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalLMDBDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalLatencyStatsDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalMapAndBatchDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalMapDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalMatchingFilesDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalMaterializedIndexDatasetHandle
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalMaxIntraOpParallelismDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalNonSerializableDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalNumaMapAndBatchDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalParallelInterleaveDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalParseExampleDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalPrivateThreadPoolDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalRandomDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalRebatchDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalScanDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalSetStatsAggregatorDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalSleepDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalSlidingWindowDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalSqlDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalStatsAggregatorHandle
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalStatsAggregatorSummary
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalTakeWhileDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalThreadPoolDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalThreadPoolHandle
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalUnbatchDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import ExperimentalUniqueDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import SamplingDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import SnapshotDataset
from tensorflow.python.ops.gen_experimental_dataset_ops import StatsAggregatorHandleV2
from tensorflow.python.ops.gen_experimental_dataset_ops import StatsAggregatorSetSummaryWriter
from tensorflow.python.ops.gen_functional_ops import Case
from tensorflow.python.ops.gen_functional_ops import FakeParam
from tensorflow.python.ops.gen_functional_ops import For
from tensorflow.python.ops.gen_functional_ops import If
from tensorflow.python.ops.gen_functional_ops import PartitionedCall
from tensorflow.python.ops.gen_functional_ops import RemoteCall
from tensorflow.python.ops.gen_functional_ops import StatefulPartitionedCall
from tensorflow.python.ops.gen_functional_ops import StatelessIf
from tensorflow.python.ops.gen_functional_ops import StatelessWhile
from tensorflow.python.ops.gen_functional_ops import SymbolicGradient
from tensorflow.python.ops.gen_functional_ops import While
from tensorflow.python.ops.gen_image_ops import AdjustContrast
from tensorflow.python.ops.gen_image_ops import AdjustContrastv2
from tensorflow.python.ops.gen_image_ops import AdjustHue
from tensorflow.python.ops.gen_image_ops import AdjustSaturation
from tensorflow.python.ops.gen_image_ops import CombinedNonMaxSuppression
from tensorflow.python.ops.gen_image_ops import CropAndResize
from tensorflow.python.ops.gen_image_ops import CropAndResizeGradBoxes
from tensorflow.python.ops.gen_image_ops import CropAndResizeGradImage
from tensorflow.python.ops.gen_image_ops import DecodeAndCropJpeg
from tensorflow.python.ops.gen_image_ops import DecodeBmp
from tensorflow.python.ops.gen_image_ops import DecodeGif
from tensorflow.python.ops.gen_image_ops import DecodeJpeg
from tensorflow.python.ops.gen_image_ops import DecodePng
from tensorflow.python.ops.gen_image_ops import DrawBoundingBoxes
from tensorflow.python.ops.gen_image_ops import DrawBoundingBoxesV2
from tensorflow.python.ops.gen_image_ops import EncodeJpeg
from tensorflow.python.ops.gen_image_ops import EncodeJpegVariableQuality
from tensorflow.python.ops.gen_image_ops import EncodePng
from tensorflow.python.ops.gen_image_ops import ExtractGlimpse
from tensorflow.python.ops.gen_image_ops import ExtractJpegShape
from tensorflow.python.ops.gen_image_ops import HSVToRGB
from tensorflow.python.ops.gen_image_ops import NonMaxSuppression
from tensorflow.python.ops.gen_image_ops import NonMaxSuppressionV2
from tensorflow.python.ops.gen_image_ops import NonMaxSuppressionV3
from tensorflow.python.ops.gen_image_ops import NonMaxSuppressionV4
from tensorflow.python.ops.gen_image_ops import NonMaxSuppressionWithOverlaps
from tensorflow.python.ops.gen_image_ops import QuantizedResizeBilinear
from tensorflow.python.ops.gen_image_ops import RGBToHSV
from tensorflow.python.ops.gen_image_ops import RandomCrop
from tensorflow.python.ops.gen_image_ops import ResizeArea
from tensorflow.python.ops.gen_image_ops import ResizeBicubic
from tensorflow.python.ops.gen_image_ops import ResizeBicubicGrad
from tensorflow.python.ops.gen_image_ops import ResizeBilinear
from tensorflow.python.ops.gen_image_ops import ResizeBilinearGrad
from tensorflow.python.ops.gen_image_ops import ResizeNearestNeighbor
from tensorflow.python.ops.gen_image_ops import ResizeNearestNeighborGrad
from tensorflow.python.ops.gen_image_ops import SampleDistortedBoundingBox
from tensorflow.python.ops.gen_image_ops import SampleDistortedBoundingBoxV2
from tensorflow.python.ops.gen_image_ops import ScaleAndTranslate
from tensorflow.python.ops.gen_image_ops import ScaleAndTranslateGrad
from tensorflow.python.ops.gen_io_ops import FixedLengthRecordReader
from tensorflow.python.ops.gen_io_ops import IdentityReader
from tensorflow.python.ops.gen_io_ops import LMDBReader
from tensorflow.python.ops.gen_io_ops import TFRecordReader
from tensorflow.python.ops.gen_io_ops import TextLineReader
from tensorflow.python.ops.gen_io_ops import WholeFileReader
from tensorflow.python.ops.gen_list_ops import EmptyTensorList
from tensorflow.python.ops.gen_list_ops import TensorListConcat
from tensorflow.python.ops.gen_list_ops import TensorListConcatLists
from tensorflow.python.ops.gen_list_ops import TensorListConcatV2
from tensorflow.python.ops.gen_list_ops import TensorListElementShape
from tensorflow.python.ops.gen_list_ops import TensorListFromTensor
from tensorflow.python.ops.gen_list_ops import TensorListGather
from tensorflow.python.ops.gen_list_ops import TensorListGetItem
from tensorflow.python.ops.gen_list_ops import TensorListLength
from tensorflow.python.ops.gen_list_ops import TensorListPopBack
from tensorflow.python.ops.gen_list_ops import TensorListPushBack
from tensorflow.python.ops.gen_list_ops import TensorListPushBackBatch
from tensorflow.python.ops.gen_list_ops import TensorListReserve
from tensorflow.python.ops.gen_list_ops import TensorListResize
from tensorflow.python.ops.gen_list_ops import TensorListScatter
from tensorflow.python.ops.gen_list_ops import TensorListScatterIntoExistingList
from tensorflow.python.ops.gen_list_ops import TensorListScatterV2
from tensorflow.python.ops.gen_list_ops import TensorListSetItem
from tensorflow.python.ops.gen_list_ops import TensorListSplit
from tensorflow.python.ops.gen_list_ops import TensorListStack
from tensorflow.python.ops.gen_logging_ops import Assert
from tensorflow.python.ops.gen_logging_ops import AudioSummary
from tensorflow.python.ops.gen_logging_ops import AudioSummaryV2
from tensorflow.python.ops.gen_logging_ops import HistogramSummary
from tensorflow.python.ops.gen_logging_ops import ImageSummary
from tensorflow.python.ops.gen_logging_ops import MergeSummary
from tensorflow.python.ops.gen_logging_ops import Print
from tensorflow.python.ops.gen_logging_ops import PrintV2
from tensorflow.python.ops.gen_logging_ops import ScalarSummary
from tensorflow.python.ops.gen_logging_ops import TensorSummary
from tensorflow.python.ops.gen_logging_ops import TensorSummaryV2
from tensorflow.python.ops.gen_logging_ops import Timestamp
from tensorflow.python.ops.gen_lookup_ops import HashTable
from tensorflow.python.ops.gen_lookup_ops import HashTableV2
from tensorflow.python.ops.gen_lookup_ops import InitializeTable
from tensorflow.python.ops.gen_lookup_ops import InitializeTableFromTextFile
from tensorflow.python.ops.gen_lookup_ops import InitializeTableFromTextFileV2
from tensorflow.python.ops.gen_lookup_ops import InitializeTableV2
from tensorflow.python.ops.gen_lookup_ops import LookupTableExport
from tensorflow.python.ops.gen_lookup_ops import LookupTableExportV2
from tensorflow.python.ops.gen_lookup_ops import LookupTableFind
from tensorflow.python.ops.gen_lookup_ops import LookupTableFindV2
from tensorflow.python.ops.gen_lookup_ops import LookupTableImport
from tensorflow.python.ops.gen_lookup_ops import LookupTableImportV2
from tensorflow.python.ops.gen_lookup_ops import LookupTableInsert
from tensorflow.python.ops.gen_lookup_ops import LookupTableInsertV2
from tensorflow.python.ops.gen_lookup_ops import LookupTableRemoveV2
from tensorflow.python.ops.gen_lookup_ops import LookupTableSize
from tensorflow.python.ops.gen_lookup_ops import LookupTableSizeV2
from tensorflow.python.ops.gen_lookup_ops import MutableDenseHashTable
from tensorflow.python.ops.gen_lookup_ops import MutableDenseHashTableV2
from tensorflow.python.ops.gen_lookup_ops import MutableHashTable
from tensorflow.python.ops.gen_lookup_ops import MutableHashTableOfTensors
from tensorflow.python.ops.gen_lookup_ops import MutableHashTableOfTensorsV2
from tensorflow.python.ops.gen_lookup_ops import MutableHashTableV2
from tensorflow.python.ops.gen_manip_ops import Roll
from tensorflow.python.ops.gen_nccl_ops import NcclAllReduce
from tensorflow.python.ops.gen_nccl_ops import NcclBroadcast
from tensorflow.python.ops.gen_nccl_ops import NcclReduce
from tensorflow.python.ops.gen_nn_ops import AvgPool
from tensorflow.python.ops.gen_nn_ops import AvgPool3D
from tensorflow.python.ops.gen_nn_ops import AvgPool3DGrad
from tensorflow.python.ops.gen_nn_ops import AvgPoolGrad
from tensorflow.python.ops.gen_nn_ops import BatchNormWithGlobalNormalization
from tensorflow.python.ops.gen_nn_ops import BatchNormWithGlobalNormalizationGrad
from tensorflow.python.ops.gen_nn_ops import BiasAdd
from tensorflow.python.ops.gen_nn_ops import BiasAddGrad
from tensorflow.python.ops.gen_nn_ops import BiasAddV1
from tensorflow.python.ops.gen_nn_ops import Conv2D
from tensorflow.python.ops.gen_nn_ops import Conv2DBackpropFilter
from tensorflow.python.ops.gen_nn_ops import Conv2DBackpropInput
from tensorflow.python.ops.gen_nn_ops import Conv3D
from tensorflow.python.ops.gen_nn_ops import Conv3DBackpropFilter
from tensorflow.python.ops.gen_nn_ops import Conv3DBackpropFilterV2
from tensorflow.python.ops.gen_nn_ops import Conv3DBackpropInput
from tensorflow.python.ops.gen_nn_ops import Conv3DBackpropInputV2
from tensorflow.python.ops.gen_nn_ops import DataFormatDimMap
from tensorflow.python.ops.gen_nn_ops import DataFormatVecPermute
from tensorflow.python.ops.gen_nn_ops import DepthwiseConv2dNative
from tensorflow.python.ops.gen_nn_ops import DepthwiseConv2dNativeBackpropFilter
from tensorflow.python.ops.gen_nn_ops import DepthwiseConv2dNativeBackpropInput
from tensorflow.python.ops.gen_nn_ops import Dilation2D
from tensorflow.python.ops.gen_nn_ops import Dilation2DBackpropFilter
from tensorflow.python.ops.gen_nn_ops import Dilation2DBackpropInput
from tensorflow.python.ops.gen_nn_ops import Elu
from tensorflow.python.ops.gen_nn_ops import EluGrad
from tensorflow.python.ops.gen_nn_ops import FractionalAvgPool
from tensorflow.python.ops.gen_nn_ops import FractionalAvgPoolGrad
from tensorflow.python.ops.gen_nn_ops import FractionalMaxPool
from tensorflow.python.ops.gen_nn_ops import FractionalMaxPoolGrad
from tensorflow.python.ops.gen_nn_ops import FusedBatchNorm
from tensorflow.python.ops.gen_nn_ops import FusedBatchNormGrad
from tensorflow.python.ops.gen_nn_ops import FusedBatchNormGradV2
from tensorflow.python.ops.gen_nn_ops import FusedBatchNormGradV3
from tensorflow.python.ops.gen_nn_ops import FusedBatchNormV2
from tensorflow.python.ops.gen_nn_ops import FusedBatchNormV3
from tensorflow.python.ops.gen_nn_ops import FusedPadConv2D
from tensorflow.python.ops.gen_nn_ops import FusedResizeAndPadConv2D
from tensorflow.python.ops.gen_nn_ops import InTopK
from tensorflow.python.ops.gen_nn_ops import InTopKV2
from tensorflow.python.ops.gen_nn_ops import L2Loss
from tensorflow.python.ops.gen_nn_ops import LRN
from tensorflow.python.ops.gen_nn_ops import LRNGrad
from tensorflow.python.ops.gen_nn_ops import LeakyRelu
from tensorflow.python.ops.gen_nn_ops import LeakyReluGrad
from tensorflow.python.ops.gen_nn_ops import LogSoftmax
from tensorflow.python.ops.gen_nn_ops import MaxPool
from tensorflow.python.ops.gen_nn_ops import MaxPool3D
from tensorflow.python.ops.gen_nn_ops import MaxPool3DGrad
from tensorflow.python.ops.gen_nn_ops import MaxPool3DGradGrad
from tensorflow.python.ops.gen_nn_ops import MaxPoolGrad
from tensorflow.python.ops.gen_nn_ops import MaxPoolGradGrad
from tensorflow.python.ops.gen_nn_ops import MaxPoolGradGradV2
from tensorflow.python.ops.gen_nn_ops import MaxPoolGradGradWithArgmax
from tensorflow.python.ops.gen_nn_ops import MaxPoolGradV2
from tensorflow.python.ops.gen_nn_ops import MaxPoolGradWithArgmax
from tensorflow.python.ops.gen_nn_ops import MaxPoolV2
from tensorflow.python.ops.gen_nn_ops import MaxPoolWithArgmax
from tensorflow.python.ops.gen_nn_ops import NthElement
from tensorflow.python.ops.gen_nn_ops import QuantizedAvgPool
from tensorflow.python.ops.gen_nn_ops import QuantizedBatchNormWithGlobalNormalization
from tensorflow.python.ops.gen_nn_ops import QuantizedBiasAdd
from tensorflow.python.ops.gen_nn_ops import QuantizedConv2D
from tensorflow.python.ops.gen_nn_ops import QuantizedConv2DAndRelu
from tensorflow.python.ops.gen_nn_ops import QuantizedConv2DAndReluAndRequantize
from tensorflow.python.ops.gen_nn_ops import QuantizedConv2DAndRequantize
from tensorflow.python.ops.gen_nn_ops import QuantizedConv2DPerChannel
from tensorflow.python.ops.gen_nn_ops import QuantizedConv2DWithBias
from tensorflow.python.ops.gen_nn_ops import QuantizedConv2DWithBiasAndRelu
from tensorflow.python.ops.gen_nn_ops import QuantizedConv2DWithBiasAndReluAndRequantize
from tensorflow.python.ops.gen_nn_ops import QuantizedConv2DWithBiasAndRequantize
from tensorflow.python.ops.gen_nn_ops import QuantizedConv2DWithBiasSignedSumAndReluAndRequantize
from tensorflow.python.ops.gen_nn_ops import QuantizedConv2DWithBiasSumAndRelu
from tensorflow.python.ops.gen_nn_ops import QuantizedConv2DWithBiasSumAndReluAndRequantize
from tensorflow.python.ops.gen_nn_ops import QuantizedDepthwiseConv2D
from tensorflow.python.ops.gen_nn_ops import QuantizedDepthwiseConv2DWithBias
from tensorflow.python.ops.gen_nn_ops import QuantizedDepthwiseConv2DWithBiasAndRelu
from tensorflow.python.ops.gen_nn_ops import QuantizedDepthwiseConv2DWithBiasAndReluAndRequantize
from tensorflow.python.ops.gen_nn_ops import QuantizedMaxPool
from tensorflow.python.ops.gen_nn_ops import QuantizedRelu
from tensorflow.python.ops.gen_nn_ops import QuantizedRelu6
from tensorflow.python.ops.gen_nn_ops import QuantizedReluX
from tensorflow.python.ops.gen_nn_ops import Relu
from tensorflow.python.ops.gen_nn_ops import Relu6
from tensorflow.python.ops.gen_nn_ops import Relu6Grad
from tensorflow.python.ops.gen_nn_ops import ReluGrad
from tensorflow.python.ops.gen_nn_ops import Selu
from tensorflow.python.ops.gen_nn_ops import SeluGrad
from tensorflow.python.ops.gen_nn_ops import Softmax
from tensorflow.python.ops.gen_nn_ops import SoftmaxCrossEntropyWithLogits
from tensorflow.python.ops.gen_nn_ops import Softplus
from tensorflow.python.ops.gen_nn_ops import SoftplusGrad
from tensorflow.python.ops.gen_nn_ops import Softsign
from tensorflow.python.ops.gen_nn_ops import SoftsignGrad
from tensorflow.python.ops.gen_nn_ops import SparseSoftmaxCrossEntropyWithLogits
from tensorflow.python.ops.gen_nn_ops import TopK
from tensorflow.python.ops.gen_nn_ops import TopKV2
from tensorflow.python.ops.gen_ragged_array_ops import RaggedGather
from tensorflow.python.ops.gen_ragged_conversion_ops import RaggedTensorFromVariant
from tensorflow.python.ops.gen_ragged_conversion_ops import RaggedTensorToSparse
from tensorflow.python.ops.gen_ragged_conversion_ops import RaggedTensorToVariant
from tensorflow.python.ops.gen_ragged_math_ops import RaggedRange
from tensorflow.python.ops.gen_resource_variable_ops import AssignAddVariableOp
from tensorflow.python.ops.gen_resource_variable_ops import AssignSubVariableOp
from tensorflow.python.ops.gen_resource_variable_ops import AssignVariableOp
from tensorflow.python.ops.gen_resource_variable_ops import ConsumeMutexLock
from tensorflow.python.ops.gen_resource_variable_ops import DestroyResourceOp
from tensorflow.python.ops.gen_resource_variable_ops import MutexLock
from tensorflow.python.ops.gen_resource_variable_ops import MutexV2
from tensorflow.python.ops.gen_resource_variable_ops import ReadVariableOp
from tensorflow.python.ops.gen_resource_variable_ops import ResourceGather
from tensorflow.python.ops.gen_resource_variable_ops import ResourceGatherNd
from tensorflow.python.ops.gen_resource_variable_ops import ResourceScatterAdd
from tensorflow.python.ops.gen_resource_variable_ops import ResourceScatterDiv
from tensorflow.python.ops.gen_resource_variable_ops import ResourceScatterMax
from tensorflow.python.ops.gen_resource_variable_ops import ResourceScatterMin
from tensorflow.python.ops.gen_resource_variable_ops import ResourceScatterMul
from tensorflow.python.ops.gen_resource_variable_ops import ResourceScatterSub
from tensorflow.python.ops.gen_resource_variable_ops import ResourceScatterUpdate
from tensorflow.python.ops.gen_resource_variable_ops import VarHandleOp
from tensorflow.python.ops.gen_resource_variable_ops import VarIsInitializedOp
from tensorflow.python.ops.gen_resource_variable_ops import VariableShape
from tensorflow.python.ops.gen_script_ops import EagerPyFunc
from tensorflow.python.ops.gen_script_ops import PyFunc
from tensorflow.python.ops.gen_script_ops import PyFuncStateless
from tensorflow.python.ops.gen_sdca_ops import SdcaFprint
from tensorflow.python.ops.gen_sdca_ops import SdcaOptimizer
from tensorflow.python.ops.gen_sdca_ops import SdcaOptimizerV2
from tensorflow.python.ops.gen_sdca_ops import SdcaShrinkL1
from tensorflow.python.ops.gen_set_ops import DenseToDenseSetOperation
from tensorflow.python.ops.gen_set_ops import DenseToSparseSetOperation
from tensorflow.python.ops.gen_set_ops import SetSize
from tensorflow.python.ops.gen_set_ops import SparseToSparseSetOperation
from tensorflow.python.ops.gen_spectral_ops import BatchFFT
from tensorflow.python.ops.gen_spectral_ops import BatchFFT2D
from tensorflow.python.ops.gen_spectral_ops import BatchFFT3D
from tensorflow.python.ops.gen_spectral_ops import BatchIFFT
from tensorflow.python.ops.gen_spectral_ops import BatchIFFT2D
from tensorflow.python.ops.gen_spectral_ops import BatchIFFT3D
from tensorflow.python.ops.gen_spectral_ops import FFT
from tensorflow.python.ops.gen_spectral_ops import FFT2D
from tensorflow.python.ops.gen_spectral_ops import FFT3D
from tensorflow.python.ops.gen_spectral_ops import IFFT
from tensorflow.python.ops.gen_spectral_ops import IFFT2D
from tensorflow.python.ops.gen_spectral_ops import IFFT3D
from tensorflow.python.ops.gen_spectral_ops import IRFFT
from tensorflow.python.ops.gen_spectral_ops import IRFFT2D
from tensorflow.python.ops.gen_spectral_ops import IRFFT3D
from tensorflow.python.ops.gen_spectral_ops import RFFT
from tensorflow.python.ops.gen_spectral_ops import RFFT2D
from tensorflow.python.ops.gen_spectral_ops import RFFT3D
from tensorflow.python.ops.gen_state_ops import Assign
from tensorflow.python.ops.gen_state_ops import AssignAdd
from tensorflow.python.ops.gen_state_ops import AssignSub
from tensorflow.python.ops.gen_state_ops import CountUpTo
from tensorflow.python.ops.gen_state_ops import DestroyTemporaryVariable
from tensorflow.python.ops.gen_state_ops import IsVariableInitialized
from tensorflow.python.ops.gen_state_ops import ResourceCountUpTo
from tensorflow.python.ops.gen_state_ops import ResourceScatterNdAdd
from tensorflow.python.ops.gen_state_ops import ResourceScatterNdSub
from tensorflow.python.ops.gen_state_ops import ResourceScatterNdUpdate
from tensorflow.python.ops.gen_state_ops import ScatterAdd
from tensorflow.python.ops.gen_state_ops import ScatterDiv
from tensorflow.python.ops.gen_state_ops import ScatterMax
from tensorflow.python.ops.gen_state_ops import ScatterMin
from tensorflow.python.ops.gen_state_ops import ScatterMul
from tensorflow.python.ops.gen_state_ops import ScatterNdAdd
from tensorflow.python.ops.gen_state_ops import ScatterNdSub
from tensorflow.python.ops.gen_state_ops import ScatterNdUpdate
from tensorflow.python.ops.gen_state_ops import ScatterSub
from tensorflow.python.ops.gen_state_ops import ScatterUpdate
from tensorflow.python.ops.gen_state_ops import TemporaryVariable
from tensorflow.python.ops.gen_state_ops import Variable
from tensorflow.python.ops.gen_state_ops import VariableV2
from tensorflow.python.ops.gen_stateful_random_ops import NonDeterministicInts
from tensorflow.python.ops.gen_stateful_random_ops import RngSkip
from tensorflow.python.ops.gen_stateful_random_ops import StatefulRandomBinomial
from tensorflow.python.ops.gen_stateful_random_ops import StatefulStandardNormal
from tensorflow.python.ops.gen_stateful_random_ops import StatefulStandardNormalV2
from tensorflow.python.ops.gen_stateful_random_ops import StatefulTruncatedNormal
from tensorflow.python.ops.gen_stateful_random_ops import StatefulUniform
from tensorflow.python.ops.gen_stateful_random_ops import StatefulUniformFullInt
from tensorflow.python.ops.gen_stateful_random_ops import StatefulUniformInt
from tensorflow.python.ops.gen_stateless_random_ops import StatelessMultinomial
from tensorflow.python.ops.gen_stateless_random_ops import StatelessRandomNormal
from tensorflow.python.ops.gen_stateless_random_ops import StatelessRandomUniform
from tensorflow.python.ops.gen_stateless_random_ops import StatelessRandomUniformInt
from tensorflow.python.ops.gen_stateless_random_ops import StatelessTruncatedNormal
from tensorflow.python.ops.gen_summary_ops import CloseSummaryWriter
from tensorflow.python.ops.gen_summary_ops import CreateSummaryDbWriter
from tensorflow.python.ops.gen_summary_ops import CreateSummaryFileWriter
from tensorflow.python.ops.gen_summary_ops import FlushSummaryWriter
from tensorflow.python.ops.gen_summary_ops import ImportEvent
from tensorflow.python.ops.gen_summary_ops import SummaryWriter
from tensorflow.python.ops.gen_summary_ops import WriteAudioSummary
from tensorflow.python.ops.gen_summary_ops import WriteGraphSummary
from tensorflow.python.ops.gen_summary_ops import WriteHistogramSummary
from tensorflow.python.ops.gen_summary_ops import WriteImageSummary
from tensorflow.python.ops.gen_summary_ops import WriteRawProtoSummary
from tensorflow.python.ops.gen_summary_ops import WriteScalarSummary
from tensorflow.python.ops.gen_summary_ops import WriteSummary
from tensorflow.python.ops.gen_tpu_ops import AllToAll
from tensorflow.python.ops.gen_tpu_ops import CollectivePermute
from tensorflow.python.ops.gen_tpu_ops import ConfigureDistributedTPU
from tensorflow.python.ops.gen_tpu_ops import CrossReplicaSum
from tensorflow.python.ops.gen_tpu_ops import EnqueueTPUEmbeddingIntegerBatch
from tensorflow.python.ops.gen_tpu_ops import EnqueueTPUEmbeddingSparseBatch
from tensorflow.python.ops.gen_tpu_ops import EnqueueTPUEmbeddingSparseTensorBatch
from tensorflow.python.ops.gen_tpu_ops import InfeedDequeue
from tensorflow.python.ops.gen_tpu_ops import InfeedDequeueTuple
from tensorflow.python.ops.gen_tpu_ops import InfeedEnqueue
from tensorflow.python.ops.gen_tpu_ops import InfeedEnqueuePrelinearizedBuffer
from tensorflow.python.ops.gen_tpu_ops import InfeedEnqueueTuple
from tensorflow.python.ops.gen_tpu_ops import LoadTPUEmbeddingADAMParameters
from tensorflow.python.ops.gen_tpu_ops import LoadTPUEmbeddingADAMParametersGradAccumDebug
from tensorflow.python.ops.gen_tpu_ops import LoadTPUEmbeddingAdadeltaParameters
from tensorflow.python.ops.gen_tpu_ops import LoadTPUEmbeddingAdadeltaParametersGradAccumDebug
from tensorflow.python.ops.gen_tpu_ops import LoadTPUEmbeddingAdagradParameters
from tensorflow.python.ops.gen_tpu_ops import LoadTPUEmbeddingAdagradParametersGradAccumDebug
from tensorflow.python.ops.gen_tpu_ops import LoadTPUEmbeddingCenteredRMSPropParameters
from tensorflow.python.ops.gen_tpu_ops import LoadTPUEmbeddingFTRLParameters
from tensorflow.python.ops.gen_tpu_ops import LoadTPUEmbeddingFTRLParametersGradAccumDebug
from tensorflow.python.ops.gen_tpu_ops import LoadTPUEmbeddingMDLAdagradLightParameters
from tensorflow.python.ops.gen_tpu_ops import LoadTPUEmbeddingMomentumParameters
from tensorflow.python.ops.gen_tpu_ops import LoadTPUEmbeddingMomentumParametersGradAccumDebug
from tensorflow.python.ops.gen_tpu_ops import LoadTPUEmbeddingProximalAdagradParameters
from tensorflow.python.ops.gen_tpu_ops import LoadTPUEmbeddingProximalAdagradParametersGradAccumDebug
from tensorflow.python.ops.gen_tpu_ops import LoadTPUEmbeddingRMSPropParameters
from tensorflow.python.ops.gen_tpu_ops import LoadTPUEmbeddingRMSPropParametersGradAccumDebug
from tensorflow.python.ops.gen_tpu_ops import LoadTPUEmbeddingStochasticGradientDescentParameters
from tensorflow.python.ops.gen_tpu_ops import OutfeedDequeue
from tensorflow.python.ops.gen_tpu_ops import OutfeedDequeueTuple
from tensorflow.python.ops.gen_tpu_ops import OutfeedEnqueue
from tensorflow.python.ops.gen_tpu_ops import OutfeedEnqueueTuple
from tensorflow.python.ops.gen_tpu_ops import Prelinearize
from tensorflow.python.ops.gen_tpu_ops import PrelinearizeTuple
from tensorflow.python.ops.gen_tpu_ops import RecvTPUEmbeddingActivations
from tensorflow.python.ops.gen_tpu_ops import RetrieveTPUEmbeddingADAMParameters
from tensorflow.python.ops.gen_tpu_ops import RetrieveTPUEmbeddingADAMParametersGradAccumDebug
from tensorflow.python.ops.gen_tpu_ops import RetrieveTPUEmbeddingAdadeltaParameters
from tensorflow.python.ops.gen_tpu_ops import RetrieveTPUEmbeddingAdadeltaParametersGradAccumDebug
from tensorflow.python.ops.gen_tpu_ops import RetrieveTPUEmbeddingAdagradParameters
from tensorflow.python.ops.gen_tpu_ops import RetrieveTPUEmbeddingAdagradParametersGradAccumDebug
from tensorflow.python.ops.gen_tpu_ops import RetrieveTPUEmbeddingCenteredRMSPropParameters
from tensorflow.python.ops.gen_tpu_ops import RetrieveTPUEmbeddingFTRLParameters
from tensorflow.python.ops.gen_tpu_ops import RetrieveTPUEmbeddingFTRLParametersGradAccumDebug
from tensorflow.python.ops.gen_tpu_ops import RetrieveTPUEmbeddingMDLAdagradLightParameters
from tensorflow.python.ops.gen_tpu_ops import RetrieveTPUEmbeddingMomentumParameters
from tensorflow.python.ops.gen_tpu_ops import RetrieveTPUEmbeddingMomentumParametersGradAccumDebug
from tensorflow.python.ops.gen_tpu_ops import RetrieveTPUEmbeddingProximalAdagradParameters
from tensorflow.python.ops.gen_tpu_ops import RetrieveTPUEmbeddingProximalAdagradParametersGradAccumDebug
from tensorflow.python.ops.gen_tpu_ops import RetrieveTPUEmbeddingRMSPropParameters
from tensorflow.python.ops.gen_tpu_ops import RetrieveTPUEmbeddingRMSPropParametersGradAccumDebug
from tensorflow.python.ops.gen_tpu_ops import RetrieveTPUEmbeddingStochasticGradientDescentParameters
from tensorflow.python.ops.gen_tpu_ops import SendTPUEmbeddingGradients
from tensorflow.python.ops.gen_tpu_ops import ShutdownDistributedTPU
from tensorflow.python.ops.gen_tpu_ops import TPUCompilationResult
from tensorflow.python.ops.gen_tpu_ops import TPUEmbeddingActivations
from tensorflow.python.ops.gen_tpu_ops import TPUOrdinalSelector
from tensorflow.python.ops.gen_tpu_ops import TPUPartitionedCall
from tensorflow.python.ops.gen_tpu_ops import TPUReplicateMetadata
from tensorflow.python.ops.gen_tpu_ops import TPUReplicatedInput
from tensorflow.python.ops.gen_tpu_ops import TPUReplicatedOutput
from tensorflow.python.ops.gen_tpu_ops import WorkerHeartbeat
from tensorflow.python.ops.gen_user_ops import Fact
from tensorflow.python.training.gen_training_ops import ApplyAdaMax
from tensorflow.python.training.gen_training_ops import ApplyAdadelta
from tensorflow.python.training.gen_training_ops import ApplyAdagrad
from tensorflow.python.training.gen_training_ops import ApplyAdagradDA
from tensorflow.python.training.gen_training_ops import ApplyAdam
from tensorflow.python.training.gen_training_ops import ApplyAddSign
from tensorflow.python.training.gen_training_ops import ApplyCenteredRMSProp
from tensorflow.python.training.gen_training_ops import ApplyFtrl
from tensorflow.python.training.gen_training_ops import ApplyFtrlV2
from tensorflow.python.training.gen_training_ops import ApplyGradientDescent
from tensorflow.python.training.gen_training_ops import ApplyMomentum
from tensorflow.python.training.gen_training_ops import ApplyPowerSign
from tensorflow.python.training.gen_training_ops import ApplyProximalAdagrad
from tensorflow.python.training.gen_training_ops import ApplyProximalGradientDescent
from tensorflow.python.training.gen_training_ops import ApplyRMSProp
from tensorflow.python.training.gen_training_ops import ResourceApplyAdaMax
from tensorflow.python.training.gen_training_ops import ResourceApplyAdadelta
from tensorflow.python.training.gen_training_ops import ResourceApplyAdagrad
from tensorflow.python.training.gen_training_ops import ResourceApplyAdagradDA
from tensorflow.python.training.gen_training_ops import ResourceApplyAdam
from tensorflow.python.training.gen_training_ops import ResourceApplyAdamWithAmsgrad
from tensorflow.python.training.gen_training_ops import ResourceApplyAddSign
from tensorflow.python.training.gen_training_ops import ResourceApplyCenteredRMSProp
from tensorflow.python.training.gen_training_ops import ResourceApplyFtrl
from tensorflow.python.training.gen_training_ops import ResourceApplyFtrlV2
from tensorflow.python.training.gen_training_ops import ResourceApplyGradientDescent
from tensorflow.python.training.gen_training_ops import ResourceApplyKerasMomentum
from tensorflow.python.training.gen_training_ops import ResourceApplyMomentum
from tensorflow.python.training.gen_training_ops import ResourceApplyPowerSign
from tensorflow.python.training.gen_training_ops import ResourceApplyProximalAdagrad
from tensorflow.python.training.gen_training_ops import ResourceApplyProximalGradientDescent
from tensorflow.python.training.gen_training_ops import ResourceApplyRMSProp
from tensorflow.python.training.gen_training_ops import ResourceSparseApplyAdadelta
from tensorflow.python.training.gen_training_ops import ResourceSparseApplyAdagrad
from tensorflow.python.training.gen_training_ops import ResourceSparseApplyAdagradDA
from tensorflow.python.training.gen_training_ops import ResourceSparseApplyCenteredRMSProp
from tensorflow.python.training.gen_training_ops import ResourceSparseApplyFtrl
from tensorflow.python.training.gen_training_ops import ResourceSparseApplyFtrlV2
from tensorflow.python.training.gen_training_ops import ResourceSparseApplyKerasMomentum
from tensorflow.python.training.gen_training_ops import ResourceSparseApplyMomentum
from tensorflow.python.training.gen_training_ops import ResourceSparseApplyProximalAdagrad
from tensorflow.python.training.gen_training_ops import ResourceSparseApplyProximalGradientDescent
from tensorflow.python.training.gen_training_ops import ResourceSparseApplyRMSProp
from tensorflow.python.training.gen_training_ops import SparseApplyAdadelta
from tensorflow.python.training.gen_training_ops import SparseApplyAdagrad
from tensorflow.python.training.gen_training_ops import SparseApplyAdagradDA
from tensorflow.python.training.gen_training_ops import SparseApplyCenteredRMSProp
from tensorflow.python.training.gen_training_ops import SparseApplyFtrl
from tensorflow.python.training.gen_training_ops import SparseApplyFtrlV2
from tensorflow.python.training.gen_training_ops import SparseApplyMomentum
from tensorflow.python.training.gen_training_ops import SparseApplyProximalAdagrad
from tensorflow.python.training.gen_training_ops import SparseApplyProximalGradientDescent
from tensorflow.python.training.gen_training_ops import SparseApplyRMSProp

del _print_function

import sys as _sys
from tensorflow.python.util import deprecation_wrapper as _deprecation_wrapper

if not isinstance(_sys.modules[__name__], _deprecation_wrapper.DeprecationWrapper):
  _sys.modules[__name__] = _deprecation_wrapper.DeprecationWrapper(
      _sys.modules[__name__], "raw_ops")
